/*
   Copyright (c) 2018 by Plexim GmbH
   All rights reserved.

   A free license is granted to anyone to use this software for any legal
   non safety-critical purpose, including commercial applications, provided
   that:
   1) IT IS NOT USED TO DIRECTLY OR INDIRECTLY COMPETE WITH PLEXIM, and
   2) THIS COPYRIGHT NOTICE IS PRESERVED in its entirety.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
   OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
   OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
   SOFTWARE.
 */

#ifndef _INCLUDES_H_
#define _INCLUDES_H_

#if !defined(_PLEXIM_) || !defined(CPU1)
#error Incorrect project settings!
#endif

#include <stdint.h>
#include <stdbool.h>
#include <stdlib.h>

#include "f28004x_device.h"
#include "f28004x_sysctrl.h"
#include "f28004x_epwm_defines.h"
#include "f28004x_pie_defines.h"
#include "f28004x_gpio_defines.h"
#include "f28004x_cputimervars.h"
#include "f28004x_globalPrototypes.h"
#include "f28004x_adc.h"
#include "f28004x_adc_defines.h"

#include "pil.h"

#define THIS_TSP_VER 0x0102

#define PLX_ASSERT(x) do {\
   if(!(x)){\
      asm("        ESTOP0");\
      for(;;);\
   }\
} while(0)

#ifndef BUILD_HAL

#ifndef BASE_NAME
#error BASE_NAME undefined.
#endif

#define xstr(x) #x
#define str(x) xstr(x)
#define sub(x) x

#include str(sub(BASE_NAME)_settings_cg.h)
#include str(sub(BASE_NAME)_probes_cg.h)

#if (PLL_SYSCLK_DIV != 1)
#error PLL_SYSCLK_DIV = 1 is currently hard-coded
#endif

#if (PLL_ODIV != 0)
#error PLL_ODIV = 0 is currently hard-coded
#endif

#if defined(SCI_BAUD_RATE) && defined(SCI_PORT)
#define SCI_CONFIGURED
#endif

#ifdef TSP_VER
#if (TSP_VER != THIS_TSP_VER)
#error TSP Version mismatch.
#endif
#endif

// Delay macro
extern void F28x_usDelay(long LoopCount);
#define PLX_DELAY_US(A)  F28x_usDelay( \
        ((((long double) A * \
           1000.0L) / \
          (long double)PLX_CPU_RATE) - 9.0L) / 5.0L)

#define PARALLEL_COM_BUF_ADDR 0x00BF00
#define PARALLEL_COM_BUF_LEN 0x80

#endif // BUILD_HAL
#endif // INCLUDES_H_
