/*
   Copyright (c) 2014-2020 by Plexim GmbH
   All rights reserved.

   A free license is granted to anyone to use this software for any legal
   non safety-critical purpose, including commercial applications, provided
   that:
   1) IT IS NOT USED TO DIRECTLY OR INDIRECTLY COMPETE WITH PLEXIM, and
   2) THIS COPYRIGHT NOTICE IS PRESERVED in its entirety.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
   OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
   OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
   SOFTWARE.
 */


#include "includes.h"

#ifndef HAL_H_
#define HAL_H_

#include "dio.h"
#include "ain.h"
#include "pwm.h"
#include "qep.h"
#include "spi.h"
#include "power.h"
#include "pga.h"
#include "dac.h"
#include "cap.h"
#include "canbus.h"

#include "hal_impl.h"

extern void HAL_sinit();
extern void HAL_configure();

// digital I/O
extern void HAL_setupDigitalOut(uint16_t aChannel, uint16_t aResourceNumber, bool aInitialValue);
extern void HAL_setupDigitalIn(uint16_t aChannel, uint16_t aResourceNumber);
extern void HAL_setDigitalOut(uint16_t aChannel, bool aVal);
extern bool HAL_getDigitalIn(uint16_t aChannel);

// analog in
extern AIN_AdcParams_t* HAL_getDefaultAdcParams();
extern void HAL_setupAdc(uint16_t aHandle, uint16_t aAdc, const AIN_AdcParams_t *aInParams);
extern AIN_ChannelParams_t* HAL_getDefaultAinChannelParams();
extern void HAL_setupAnalogIn(uint16_t aHandle, uint16_t aChannel, uint16_t aResourceNumber,  const AIN_ChannelParams_t *aParams);
extern void HAL_setupAnalogInF(uint16_t aHandle, uint16_t aChannel, uint16_t aResourceNumber, const AIN_ChannelParams_t *aParams, float scale, float offset);
extern uint16_t HAL_getAnalogIn(uint16_t aHandle, uint16_t aChannel);
extern float HAL_getAnalogInF(uint16_t aHandle, uint16_t aChannel);

// PWM
extern PWM_Params_t* HAL_getDefaultPwmParams();
extern void HAL_setupPwm(uint16_t aChannel, uint16_t aResourceNumber, const PWM_Params_t *aPwmParams);
extern void HAL_setPwm(uint16_t aChannel, uint16_t aVal);
extern void HAL_setPwmDuty(uint16_t aChannel, float aDuty);
extern void HAL_setPwmSequence(uint16_t aChannel, uint16_t aSequence);
extern void HAL_setPwmDutyFreqPhase(uint16_t aChannel, float aDuty, float aFreqScaling, float aPhase);
extern uint32_t HAL_getPwmCounter(uint16_t aChannel);
extern bool HAL_getPwmCountDirection(uint16_t aChannel);
extern void HAL_setupPwmSyncSelect(uint16_t aResourceNumber, uint16_t aType, uint16_t aUnit);
// experimental (do not use for released target blocks)
extern void HAL_setPwmToPassive(uint16_t aChannel);
extern void HAL_setPwmToOperational(uint16_t aChannel);
extern void HAL_setPwmDeadtimeCounts(uint16_t aChannel, uint16_t aRisingEdgeDelay, uint16_t aFallingEdgeDelay);

// powerstage
extern void HAL_enablePower(bool aEnable);
extern bool HAL_powerIsEnabled();
extern void HAL_setupPwrTzGpio(uint16_t aTzId, uint16_t aGpio);

// QEP
extern QEP_Params_t* HAL_getDefaultQepParams(uint32_t aPeriodCounts, bool aEnableResetByIndexPulse);
extern void HAL_setupQep(uint16_t aChannel, uint16_t aResourceNumber, uint16_t aPinSet, QEP_Params_t *aParams);
extern uint32_t HAL_getQepCounter(uint16_t aChannel);
extern bool HAL_getAndClearQepIndexFlag(uint16_t aChannel);
extern uint32_t HAL_getQepIndexLatchCounter(uint16_t aChannel);

// PGA
extern void HAL_enablePga(uint16_t aChannel, float aGain, float aFilterResistor);

// DAC
extern void HAL_setupDac(uint16_t aChannel, uint16_t aDac, float aScale, float aOffset, float aMin, float aMax);
extern void HAL_setDacOutF(uint16_t aChannel, float aVal);

// CAP - experimental (do not use for released target blocks)
extern CAP_Params_t* HAL_getDefaultCapParams(uint16_t aNumEvents, bool aSingleShotOperation);
extern void HAL_setupCap(uint16_t aChannel, uint16_t aResourceNumber, uint16_t aGpio, CAP_Params_t *aParams);
extern bool HAL_getNewCapValues(uint16_t aChannel, uint16_t aNewPrescale, uint32_t *aValues, bool *aOverflowFlag);

// SPI - experimental (do not use for released target blocks)
extern SPI_Params_t* HAL_getDefaultSpiParams(uint16_t aCharLength, bool aPol, bool aPhase, uint32_t aBaudRate, bool aMasterSlave);
extern void HAL_setupSpi(uint16_t aChannel, uint16_t aResourceNumber, uint16_t aPinSet, SPI_Params_t *aParams);
extern uint16_t HAL_getSpiRxFifoLevel(int16_t aChannel);
extern bool HAL_getAndResetSpiRxOverrunFlag(int16_t aChannel);
extern bool HAL_putSpiWords(int16_t aChannel, uint16_t *aData, uint16_t aLen);
extern bool HAL_getSpiWords(int16_t aChannel, uint16_t *aData, uint16_t aLen);

// CAN - experimental (do not use for released target blocks)
extern CANBUS_Params_t* HAL_getDefaultCanParams();
extern void HAL_setupCan(uint16_t aChannel, uint16_t aResourceNumber, uint16_t aPinSet, CANBUS_Params_t *aParams);
extern void HAL_setupMailbox(uint16_t aChannel, uint16_t aMailBox, bool aConfigureForTransmit, uint32_t aId, bool isExtended, unsigned char aLen);
extern bool HAL_getCanMessage(uint16_t aChannel, uint16_t aMailBox, unsigned char data[], unsigned char lenMax);
extern bool HAL_putCanMessage(uint16_t aChannel, uint16_t aMailBox, unsigned char data[], unsigned char len);

extern void HAL_setCanBusOn(uint16_t aChannel, bool aBusOn);
extern bool HAL_getCanIsBusOn(uint16_t aChannel);
extern bool HAL_getCanIsErrorActive(uint16_t aChannel);

#endif /* HAL_H_ */
