/*
   Copyright (c) 2014-2016 by Plexim GmbH
   All rights reserved.

   A free license is granted to anyone to use this software for any legal
   non safety-critical purpose, including commercial applications, provided
   that:
   1) IT IS NOT USED TO DIRECTLY OR INDIRECTLY COMPETE WITH PLEXIM, and
   2) THIS COPYRIGHT NOTICE IS PRESERVED in its entirety.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
   OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
   OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
   SOFTWARE.
 */

#ifndef PWM_IMPL_H_
#define PWM_IMPL_H_

typedef enum PWM_UNIT {
	PWM_EPWM_NONE=0,
	PWM_EPWM1,
	PWM_EPWM2,
	PWM_EPWM3,
	PWM_EPWM4,
	PWM_EPWM5,
	PWM_EPWM6,
	PWM_EPWM7,
	PWM_EPWM8
} PWM_Unit_t;

typedef struct PWM_REG_PARAMS {
    Uint16 TBPRD;
    union TBPHS_REG  TBPHS;
    union TBCTL_REG TBCTL;
    union CMPCTL_REG CMPCTL;
    union AQCTLA_REG AQCTLA;
    union DBCTL_REG DBCTL;
    union DBFED_REG DBFED;
    union DBRED_REG DBRED;
    union TZCTL_REG TZCTL;
    union ETSEL_REG ETSEL;
    union TZSEL_REG TZSEL;
    union ETSOCPS_REG ETSOCPS;
} PWM_RegParams_t;

typedef enum PWM_OUTPUT_MODE {
    PWM_OUTPUT_MODE_DUAL=0,
    PWM_OUTPUT_MODE_SINGLE,
    PWM_OUTPUT_MODE_DISABLED
} PWM_OutputMode_t;

typedef struct PWM_PARAMS {
    PWM_RegParams_t reg;
    PWM_OutputMode_t outMode;
} PWM_Params_t;

typedef struct PWM_OBJ
{
	uint16_t *cmprProbe;
	volatile struct EPWM_REGS *pwm;
	uint16_t nomTBPRD;
    uint16_t sequence;
} PWM_Obj_t;

typedef PWM_Obj_t *PWM_Handle_t;

extern void PWM_getRegisterBase(PWM_Unit_t aPwmChannel, volatile struct EPWM_REGS** aReg);

inline uint32_t PWM_getFullDutyCompare(PWM_Handle_t aHandle)
{
	PWM_Obj_t *obj = (PWM_Obj_t *)aHandle;
	if(obj->pwm->TBCTL.bit.CTRMODE == TB_COUNT_UPDOWN)
	{
	    // up-down
	    return obj->pwm->TBPRD;
	}
	else
	{
	    // sawtooth
	    return obj->pwm->TBPRD+1;
	}
}

inline uint32_t PWM_getCounter(PWM_Handle_t aHandle)
{
	PWM_Obj_t *obj = (PWM_Obj_t *)aHandle;
	return obj->pwm->TBCTR;
}

inline bool PWM_getCountDirection(PWM_Handle_t aHandle)
{
	PWM_Obj_t *obj = (PWM_Obj_t *)aHandle;
	return obj->pwm->TBSTS.bit.CTRDIR;
}

inline void PWM_enableOut(PWM_Handle_t aHandle)
{
	PWM_Obj_t *obj = (PWM_Obj_t *)aHandle;
	EALLOW;
	obj->pwm->TZCLR.bit.OST=1;
	EDIS;
}

inline void PWM_disableOut(PWM_Handle_t aHandle)
{
	PWM_Obj_t *obj = (PWM_Obj_t *)aHandle;
	EALLOW;
	obj->pwm->TZFRC.bit.OST=1;
	EDIS;
}

inline void PWM_setCompare(PWM_Handle_t aHandle, uint16_t aCompare)
{
    PWM_Obj_t *obj = (PWM_Obj_t *)aHandle;
    obj->pwm->CMPA.bit.CMPA = aCompare;
}

inline void PWM_setTZSafe(PWM_Handle_t aHandle, uint16_t aSafe)
{
	PWM_Obj_t *obj = (PWM_Obj_t *)aHandle;
	obj->pwm->TZCTL.bit.TZA = aSafe;
	obj->pwm->TZCTL.bit.TZB = aSafe;
}

inline void PWM_setComparePh(PWM_Handle_t aHandle, uint16_t aCompare, uint16_t aPh, uint16_t aDir)
{
    PWM_Obj_t *obj = (PWM_Obj_t *)aHandle;
    obj->pwm->CMPA.bit.CMPA = aCompare;
    obj->pwm->TBPHS.bit.TBPHS= aPh;
    obj->pwm->TBCTL.bit.PHSDIR = aDir;
}

inline void PWM_setDeadTimeCounts(PWM_Handle_t aHandle, uint16_t aRisingEdgeDelay, uint16_t aFallingEdgeDelay)
{
    PWM_Obj_t *obj = (PWM_Obj_t *)aHandle;
    obj->pwm->DBFED.all = aFallingEdgeDelay;
    obj->pwm->DBRED.all = aRisingEdgeDelay;
}

inline void PWM_setOutToPassive(PWM_Handle_t aHandle)
{
    PWM_Obj_t *obj = (PWM_Obj_t *)aHandle;

    obj->pwm->AQCSFRC.bit.CSFA = 1; // force low

    if(obj->pwm->DBCTL.bit.POLSEL == 1)
    {
        // DB_ACTV_LOC
        obj->pwm->AQCSFRC.bit.CSFB = 2; // force high
    }
    else
    {
        // DB_ACTV_HIC
        obj->pwm->AQCSFRC.bit.CSFB = 1; // force low
    }
    // switch complementary output over to channel B
    obj->pwm->DBCTL.bit.OUT_MODE = 0x2;
}

inline void PWM_setOutToOperational(PWM_Handle_t aHandle)
{
    PWM_Obj_t *obj = (PWM_Obj_t *)aHandle;
    obj->pwm->DBCTL.bit.OUT_MODE = 0x3;  // enable Dead-band module
    obj->pwm->AQCSFRC.bit.CSFA = 0;
}

inline void PWM_setSequence(PWM_Handle_t aHandle, uint16_t aSequence)
{
    PWM_Obj_t *obj = (PWM_Obj_t *)aHandle;
    switch(aSequence)
    {
        case 0: //negative sequence w/ forcing ZERO/PRD state
            obj->pwm->AQCTLA.bit.CAU = 2;
            obj->pwm->AQCTLA.bit.CAD = 1;
            obj->pwm->AQCTLA.bit.ZRO = 1;
            obj->pwm->AQCTLA.bit.PRD = 2;
            break;

        case 1: //positive sequence w/ forcing ZERO/PRD state
            obj->pwm->AQCTLA.bit.CAU = 1;
            obj->pwm->AQCTLA.bit.CAD = 2;
            obj->pwm->AQCTLA.bit.ZRO = 2;
            obj->pwm->AQCTLA.bit.PRD = 1;
            break;

        case 2: //negative sequence w/o forcing ZERO/PRD state
            obj->pwm->AQCTLA.bit.CAU = 2;
            obj->pwm->AQCTLA.bit.CAD = 1;
            obj->pwm->AQCTLA.bit.ZRO = 0;
            obj->pwm->AQCTLA.bit.PRD = 0;
            break;

        case 3: //positive sequence w/o forcing ZERO/PRD state
            obj->pwm->AQCTLA.bit.CAU = 1;
            obj->pwm->AQCTLA.bit.CAD = 2;
            obj->pwm->AQCTLA.bit.ZRO = 0;
            obj->pwm->AQCTLA.bit.PRD = 0;
            break;

        default:
            PLX_ASSERT(0);
    }
    obj->sequence = aSequence;
 }

inline uint16_t PWM_getSequence(PWM_Handle_t aHandle)
{
    PWM_Obj_t *obj = (PWM_Obj_t *)aHandle;
    return obj->sequence;
}

#endif /* PWM_IMPL_H_ */
