/*
   Copyright (c) 2014-2020 by Plexim GmbH
   All rights reserved.

   A free license is granted to anyone to use this software for any legal
   non safety-critical purpose, including commercial applications, provided
   that:
   1) IT IS NOT USED TO DIRECTLY OR INDIRECTLY COMPETE WITH PLEXIM, and
   2) THIS COPYRIGHT NOTICE IS PRESERVED in its entirety.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
   OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
   OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
   SOFTWARE.
 */

#include "includes.h"
#include "pil.h"
#include "pwm.h"
#include "ain.h"

#ifndef DISPATCHER_H_
#define DISPATCHER_H_

typedef void(*DISPR_TaskPtr_t)(void * const);
typedef void(*DISPR_IdleTaskPtr_t)();

#include "dispatcher_impl.h" // implementation specific

extern void DISPR_sinit();

extern void DISPR_configure(uint32_t aPeriodInSysClkTicks, PIL_Handle_t aPilHandle, DISPR_TaskObj_t *aTskMemory, uint16_t aNumTasks);
extern void DISPR_registerTask(uint16_t aTaskId, DISPR_TaskPtr_t aTsk, uint32_t aPeriodInSysClkTicks, void * const aParameters);
extern void DISPR_registerIdleTask(DISPR_IdleTaskPtr_t aTsk);

extern void DISPR_setupTimer(uint16_t aUnit, uint32_t aPeriodInSysClkTicks);
extern void DISPR_setTriggerByTimer();
extern void DISPR_setTriggerByPwm(PWM_Unit_t aPwmUnit, uint16_t aEventSel);
extern void DISPR_setTriggerByAdcViaTimer(AIN_Unit_t aAdcUnit, uint16_t aEocChannel);
extern void DISPR_setTriggerByAdcViaPwm(AIN_Unit_t aAdcUnit, uint16_t aEocChannel, PWM_Unit_t aPwmUnit);

extern void DISPR_setPowerupDelay(uint16_t aDelayInTask1Ticks);

extern void DISPR_start();

extern void DISPR_stop();

extern void DISPR_reset();

extern uint32_t DISPR_getTimeStamp();
extern float DISPR_getCpuLoad();

#endif /* DISPATCHER_H_ */
