/*
   Copyright (c) 2019 by Plexim GmbH
   All rights reserved.

   A free license is granted to anyone to use this software for any legal
   non safety-critical purpose, including commercial applications, provided
   that:
   1) IT IS NOT USED TO DIRECTLY OR INDIRECTLY COMPETE WITH PLEXIM, and
   2) THIS COPYRIGHT NOTICE IS PRESERVED in its entirety.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
   OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
   OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
   SOFTWARE.
 */

#include "dac.h"

#pragma diag_suppress 112 // PLX_ASSERT(0) in switch statements

void DAC_sinit()
{
    EALLOW;
    CpuSysRegs.PCLKCR16.bit.DAC_A = 1;
    CpuSysRegs.PCLKCR16.bit.DAC_B = 1;
    EDIS;
}

DAC_Handle_t DAC_init(void *aMemory, const size_t aNumBytes)
{
    DAC_Handle_t handle;

    if(aNumBytes < sizeof(DAC_Obj_t))
        return((DAC_Handle_t)NULL);

    // set handle
    handle = (DAC_Handle_t)aMemory;

    return handle;
}

void DAC_configure(DAC_Handle_t aHandle,  DAC_Unit_t aUnit)
{
    DAC_Obj_t *obj = (DAC_Obj_t *)aHandle;

    switch(aUnit)
    {
        case DAC_A:
            obj->dac = &DacaRegs;
            break;
        case DAC_B:
            obj->dac = &DacbRegs;
            break;
        default:
            PLX_ASSERT(0);
    }
    EALLOW;
    obj->dac->DACCTL.bit.MODE = 1; // 2x -> range 0..3.3V
    obj->dac->DACCTL.bit.DACREFSEL = 1; // REFERENCE_VDAC
    obj->dac->DACVALS.all = 0;
    obj->dac->DACOUTEN.bit.DACOUTEN = 1;
    EDIS;
}
