/*
   Copyright (c) 2014-2016 by Plexim GmbH
   All rights reserved.

   A free license is granted to anyone to use this software for any legal
   non safety-critical purpose, including commercial applications, provided
   that:
   1) IT IS NOT USED TO DIRECTLY OR INDIRECTLY COMPETE WITH PLEXIM, and
   2) THIS COPYRIGHT NOTICE IS PRESERVED in its entirety.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
   OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
   OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
   SOFTWARE.
 */

#ifndef AIN_IMPL_H_
#define AIN_IMPL_H_

#define AIN_NUM_CHANNELS 16

typedef enum AIN_UNIT {
	AIN_ADC=0
} AIN_Unit_t;

/*Dummy registers from other C2000 targets*/
struct ADCSOCxCTL_BITS {     	// bits description
	Uint16	ACQPS:6;			// 5:0	Acquisition Pulse Size
	Uint16	CHSEL:4;			// 9:6	SOCx Channel Select
	Uint16	rsvd1:1;			// 10	Reserved
	Uint16	TRIGSEL:5;			// 15:11	SOCx Trigger Select
};

union ADCSOCxCTL_REG {
	Uint16 all;
	struct ADCSOCxCTL_BITS	bit;
};

struct INTSEL1N2_BITS {     	// bits description
	Uint16	INT1SEL:5;			// 4:0	INT1 EOC Source Select
	Uint16	INT1E:1;			// 5	INT1 Interrupt Enable
	Uint16	INT1CONT:1;			// 6	INT1 Continuous Mode Enable
	Uint16	rsvd1:1;			// 7	Reserved
	Uint16	INT2SEL:5;			// 12:8	INT2 EOC Source Select
	Uint16	INT2E:1;			// 13	INT2 Interrupt Enable
	Uint16	INT2CONT:1;			// 14	INT2 Continuous Mode Enable
	Uint16	rsvd2:1;			// 15	Reserved
};

union INTSEL1N2_REG {
	Uint16 all;
	struct INTSEL1N2_BITS	bit;
};
/*Dummy end*/

typedef struct AIN_ADC_PARAMS {
	union INTSEL1N2_REG INTSEL1N2; // Dummy
	union ADCTRL1_REG ADCTRL1;
	union ADCTRL2_REG ADCTRL2;
	union ADCTRL3_REG ADCTRL3;
	union ADCREFSEL_REG ADCREFSEL;
    uint16_t dummy;
} AIN_AdcParams_t;

typedef struct AIN_OBJ
{
	volatile struct ADC_REGS *adc;
} AIN_Obj_t;

typedef struct AIN_CHANNEL_PARAMS {
	union ADCSOCxCTL_REG ADCSOCxCTL; // Dummy
} AIN_ChannelParams_t;

typedef AIN_Obj_t *AIN_Handle_t;

extern void AIN_getRegisterBase(AIN_Unit_t aAdcUnit, volatile struct ADC_REGS** aReg);

inline uint16_t AIN_getIn(AIN_Handle_t aHandle, uint16_t aChannel)
{
    AIN_Obj_t *obj = (AIN_Obj_t *)aHandle;

    PLX_ASSERT(aChannel < AIN_NUM_CHANNELS);

    uint16_t adc_result;

    switch(aChannel)
    {
    	case 0:
    		adc_result = obj->adc->ADCRESULT0 >> 4;
    		break;
    	case 1:
    		adc_result = obj->adc->ADCRESULT1 >> 4;
			break;
    	case 2:
    		adc_result = obj->adc->ADCRESULT2 >> 4;
			break;
    	case 3:
    		adc_result = obj->adc->ADCRESULT3 >> 4;
			break;
    	case 4:
    		adc_result = obj->adc->ADCRESULT4 >> 4;
			break;
    	case 5:
    		adc_result = obj->adc->ADCRESULT5 >> 4;
			break;
    	case 6:
    		adc_result = obj->adc->ADCRESULT6 >> 4;
			break;
    	case 7:
    		adc_result = obj->adc->ADCRESULT7 >> 4;
			break;
    	case 8:
    		adc_result = obj->adc->ADCRESULT8 >> 4;
			break;
    	case 9:
    		adc_result = obj->adc->ADCRESULT9 >> 4;
			break;
    	case 10:
    		adc_result = obj->adc->ADCRESULT10 >> 4;
			break;
    	case 11:
    		adc_result = obj->adc->ADCRESULT11 >> 4;
			break;
    	case 12:
    		adc_result = obj->adc->ADCRESULT12 >> 4;
			break;
    	case 13:
    		adc_result = obj->adc->ADCRESULT13 >> 4;
			break;
    	case 14:
    		adc_result = obj->adc->ADCRESULT14 >> 4;
			break;
    	case 15:
    		adc_result = obj->adc->ADCRESULT15 >> 4;
			break;
    	default:
    		break;
    }
    return adc_result;
}

#endif /* AIN_IMPL_H_ */
