/*
   Copyright (c) 2014-2019 by Plexim GmbH
   All rights reserved.

   A free license is granted to anyone to use this software for any legal
   non safety-critical purpose, including commercial applications, provided
   that:
   1) IT IS NOT USED TO DIRECTLY OR INDIRECTLY COMPETE WITH PLEXIM, and
   2) THIS COPYRIGHT NOTICE IS PRESERVED in its entirety.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
   OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
   OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
   SOFTWARE.
 */

#include "power.h"

#ifndef HAL_IMPL_H_
#define HAL_IMPL_H_

#ifndef HAL_NUM_TZ_CHANNELS
#define HAL_NUM_TZ_CHANNELS 3
#endif

#define HAL_NUM_CHANNELS_PER_ADC AIN_NUM_CHANNELS

#define HAL_VOLTS_PER_ADC_BIT (3.0/4096.0) //VREFHI 3.3V for 28069

#ifndef HAL_NUM_CAP_CHANNELS
#define HAL_NUM_CAP_CHANNELS 0
#endif

#ifndef HAL_NUM_CANBUS_CHANNELS
#define HAL_NUM_CANBUS_CHANNELS 0
#endif

typedef struct HAL_SOBJ
{
#ifndef PIL_PREP_TOOL
	DIO_Handle_t inPortHandles[HAL_NUM_DIN_CHANNELS];
	DIO_Handle_t outPortHandles[HAL_NUM_DOUT_CHANNELS];
	PWM_Handle_t pwmHandles[HAL_NUM_PWM_CHANNELS];
	QEP_Handle_t qepHandles[HAL_NUM_QEP_CHANNELS];
	AIN_Handle_t aInHandle[HAL_NUM_ADCS];
	SPI_Handle_t spiHandles[HAL_NUM_SPI_CHANNELS];
    CAP_Handle_t capHandles[HAL_NUM_CAP_CHANNELS];
    CANBUS_Handle_t canHandles[HAL_NUM_CANBUS_CHANNELS];

	DIO_Obj_t inPortObjs[HAL_NUM_DIN_CHANNELS];
	DIO_Obj_t outPortObjs[HAL_NUM_DOUT_CHANNELS];
	PWM_Obj_t pwmObjs[HAL_NUM_PWM_CHANNELS];
	QEP_Obj_t qepObjs[HAL_NUM_QEP_CHANNELS];
	AIN_Obj_t aInObjs[HAL_NUM_ADCS];
	SPI_Obj_t spiObjs[HAL_NUM_SPI_CHANNELS];
    CAP_Obj_t capObjs[HAL_NUM_CAP_CHANNELS];
    CANBUS_Obj_t canObjs[HAL_NUM_CANBUS_CHANNELS];
#endif
	uint32_t sysClkHz;
	float aInScale[HAL_NUM_ADCS][HAL_NUM_CHANNELS_PER_ADC];
	float aInOffset[HAL_NUM_ADCS][HAL_NUM_CHANNELS_PER_ADC];
	AIN_AdcParams_t aInParamsMemory;
	AIN_ChannelParams_t aInChannelParamsMemory;
	PWM_Params_t pwmParamsMemory;
    QEP_Params_t qepParamsMemory;
	SPI_Params_t spiParamsMemory;
	CAP_Params_t capParamsMemory;
    CANBUS_Params_t canParamsMemory;
} HAL_SObj_t;

extern HAL_SObj_t HAL_SObj;

inline uint16_t HAL_getAnalogIn(uint16_t aAdc, uint16_t aChannel)
{
	return AIN_getIn(HAL_SObj.aInHandle[aAdc], aChannel);
}

inline float HAL_getAnalogInF(uint16_t aAdc, uint16_t aChannel)
{
	float adcV = (float)AIN_getIn(HAL_SObj.aInHandle[aAdc], aChannel)*HAL_VOLTS_PER_ADC_BIT;
	return (adcV * HAL_SObj.aInScale[aAdc][aChannel]) + HAL_SObj.aInOffset[aAdc][aChannel];
}

inline bool HAL_getDigitalIn(uint16_t aChannel)
{
	return DIO_get(HAL_SObj.inPortHandles[aChannel]);
}

inline void HAL_setDigitalOut(uint16_t aChannel, bool aVal)
{
	DIO_set(HAL_SObj.outPortHandles[aChannel], aVal);
}

inline uint32_t HAL_getPwmCounter(uint16_t aChannel)
{
	return PWM_getCounter(HAL_SObj.pwmHandles[aChannel]);
}

inline bool HAL_getPwmCountDirection(uint16_t aChannel)
{
	return PWM_getCountDirection(HAL_SObj.pwmHandles[aChannel]);
}

inline void HAL_setPwm(uint16_t aChannel, uint16_t aVal)
{
	PWM_setCompare(HAL_SObj.pwmHandles[aChannel], aVal);
}

inline void HAL_setPwmDuty(uint16_t aChannel, float aDuty)
{
    float duty = aDuty;

    if (duty > 1.0)
    {
        duty = 1.0;
    }
    else if(duty < 0.0)
    {
        duty = 0.0;
    }

    if((PWM_getSequence(HAL_SObj.pwmHandles[aChannel]) & 1) == 0)
    {
        duty = 1.0 - duty;
    }

    uint16_t cmp = (uint16_t)(duty * (float)PWM_getFullDutyCompare(HAL_SObj.pwmHandles[aChannel]));
    PWM_setCompare(HAL_SObj.pwmHandles[aChannel], cmp);
}

inline void HAL_setPwmToPassive(uint16_t aChannel)
{
    PWM_setOutToPassive(HAL_SObj.pwmHandles[aChannel]);
}

inline void HAL_setPwmToOperational(uint16_t aChannel)
{
    PWM_setOutToOperational(HAL_SObj.pwmHandles[aChannel]);
}

inline void HAL_setPwmSequence(uint16_t aChannel, uint16_t aSequence)
{
    PWM_setSequence(HAL_SObj.pwmHandles[aChannel], aSequence);
}

inline void HAL_setPwmDutyFreqPhase(uint16_t aChannel, float aDuty, float aFreqScaling, float aPhase)
{
    if((aFreqScaling <= (1.0/65536.0)) || (aFreqScaling > 65536.0))
    {
        return; // silent user error better than assert
    }

    PWM_scalePeriod(HAL_SObj.pwmHandles[aChannel], 1.0/aFreqScaling);
    PWM_setPhase(HAL_SObj.pwmHandles[aChannel], aPhase);
    HAL_setPwmDuty(aChannel, aDuty);
}

inline void HAL_setPwmDeadtimeCounts(uint16_t aChannel, uint16_t aRisingEdgeDelay, uint16_t aFallingEdgeDelay)
{
    PWM_setDeadTimeCounts(HAL_SObj.pwmHandles[aChannel], aRisingEdgeDelay, aFallingEdgeDelay);
}

inline uint32_t HAL_getQepCounter(uint16_t aChannel)
{
	return QEP_getPosCnt(HAL_SObj.qepHandles[aChannel]);
}

inline bool HAL_getAndClearQepIndexFlag(uint16_t aChannel)
{
	return QEP_getAndClearIndexFlag(HAL_SObj.qepHandles[aChannel]);
}

inline uint32_t HAL_getQepIndexLatchCounter(uint16_t aChannel)
{
	return QEP_getPosILatchCnt(HAL_SObj.qepHandles[aChannel]);
}

inline uint16_t HAL_getSpiRxFifoLevel(int16_t aChannel)
{
    return SPI_getRxFifoLevel(HAL_SObj.spiHandles[aChannel]);
}

inline bool HAL_putSpiWords(int16_t aChannel, uint16_t *aData, uint16_t aLen)
{
    return SPI_putWords(HAL_SObj.spiHandles[aChannel], aData, aLen);
}

inline bool HAL_getSpiWords(int16_t aChannel, uint16_t *aData, uint16_t aLen)
{
    return SPI_getWords(HAL_SObj.spiHandles[aChannel], aData, aLen);
}

inline bool HAL_getAndResetSpiRxOverrunFlag(int16_t aChannel)
{
    return SPI_getAndResetRxOverrunFlag(HAL_SObj.spiHandles[aChannel]);
}

inline void HAL_setPwmPhase(uint16_t aChannel, uint16_t aVal, uint16_t aPh, uint16_t aDir)
{
	PWM_setComparePh(HAL_SObj.pwmHandles[aChannel], aVal, aPh, aDir);
}

inline void HAL_enablePower(bool aEnable)
{
    PWR_enable(aEnable);
}

inline bool HAL_powerIsEnabled()
{
    return PWR_isEnabled();
}

inline bool HAL_getNewCapValues(uint16_t aChannel, uint16_t aNewPrescale, uint32_t *aValues, bool *aOverflowFlag)
{
    return CAP_getNewValues(HAL_SObj.capHandles[aChannel], aNewPrescale, aValues, aOverflowFlag);
}

inline bool HAL_getCanMessage(uint16_t aChannel, uint16_t aMailBox, unsigned char data[], unsigned char lenMax)
{
    return CANBUS_getMessage(HAL_SObj.canHandles[aChannel], aMailBox, data, lenMax);
}

inline bool HAL_putCanMessage(uint16_t aChannel, uint16_t aMailBox, unsigned char data[], unsigned char len)
{
    return CANBUS_putMessage(HAL_SObj.canHandles[aChannel], aMailBox, data, len);
}

inline void HAL_setCanBusOn(uint16_t aChannel, bool aBusOn){
    CANBUS_setBusOn(HAL_SObj.canHandles[aChannel], aBusOn);
}

inline bool HAL_getCanIsBusOn(uint16_t aChannel){
    return CANBUS_isBusOn(HAL_SObj.canHandles[aChannel]);
}

inline bool HAL_getCanIsErrorActive(uint16_t aChannel){
    return CANBUS_isErrorActive(HAL_SObj.canHandles[aChannel]);
}

#endif /* HAL_IMPL_H_ */
