/*
   Copyright (c) 2014-2016 by Plexim GmbH
   All rights reserved.

   A free license is granted to anyone to use this software for any legal
   non safety-critical purpose, including commercial applications, provided
   that:
   1) IT IS NOT USED TO DIRECTLY OR INDIRECTLY COMPETE WITH PLEXIM, and
   2) THIS COPYRIGHT NOTICE IS PRESERVED in its entirety.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
   OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
   OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
   SOFTWARE.
 */

#include "qep.h"

QEP_Handle_t QEP_init(void *aMemory, const size_t aNumBytes)
{
	QEP_Handle_t handle;

	if(aNumBytes < sizeof(QEP_Obj_t))
		return((QEP_Handle_t)NULL);

	// set handle
	handle = (QEP_Handle_t)aMemory;

	return handle;
}

void QEP_setDefaultParams(QEP_Params_t *aParams)
{
    aParams->QPOSINIT = 0;   // set counter initialization
    aParams->QPOSMAX = 0xFFFFFFFF; // maximum count value

    aParams->QEPCTL.all = 0;
    aParams->QEPCTL.bit.FREE_SOFT = 0; // all QEP timers stop on emulation suspend
    aParams->QEPCTL.bit.PCRM = 1; // operate QEP in Reset on Max counter mode
    aParams->QEPCTL.bit.SEI = 0; // disable strobe init
    aParams->QEPCTL.bit.IEI = 0; // disable index init
    aParams->QEPCTL.bit.SWI = 0; // don't allow software to initialize counter
    aParams->QEPCTL.bit.SEL = 0; // disable strobe event
    aParams->QEPCTL.bit.IEL = 3; // latch on index event marker (only applicable for QEPCTL[PCRM]=01)
    aParams->QEPCTL.bit.QPEN = 1; // QEP enable
    aParams->QEPCTL.bit.QCLM = 0; // disable EQEP capture latch
    aParams->QEPCTL.bit.UTE = 0; // disable unit timer
    aParams->QEPCTL.bit.WDE = 0; // disable QEP watchdog
}

void QEP_configure(QEP_Handle_t aHandle, uint16_t aUnit, uint16_t aPinSet, const QEP_Params_t *aParams)
{
	QEP_Obj_t *obj = (QEP_Obj_t *)aHandle;

//#if 0
	if((aUnit < 1) || (aUnit > 3))
	{
		aUnit = 1;
	}

	EALLOW;

	switch(aUnit)
	{
		default:
		case 1:
			obj->qep = &EQep1Regs;
			CpuSysRegs.PCLKCR4.bit.EQEP1 = 1;
			switch(aPinSet)
			{
				default:
				case 10:
					// enable internal pull-up for the eQEP selected pins
					GpioCtrlRegs.GPAPUD.bit.GPIO20 = 0;
					GpioCtrlRegs.GPAPUD.bit.GPIO21 = 0;
					GpioCtrlRegs.GPDPUD.bit.GPIO99 = 0;

					// synchronize inputs to SYSCLKOUT
					GpioCtrlRegs.GPAQSEL2.bit.GPIO20 = 0;
					GpioCtrlRegs.GPAQSEL2.bit.GPIO21 = 0;
					GpioCtrlRegs.GPDQSEL1.bit.GPIO99 = 0;

					// set muxes
					GpioCtrlRegs.GPAGMUX2.bit.GPIO20 = 0;
					GpioCtrlRegs.GPAMUX2.bit.GPIO20 = 1;// 0=GPIO,  1=EQEPA-1,  2=Resv,  3=COMP1OUT
					GpioCtrlRegs.GPAGMUX2.bit.GPIO21 = 0;
					GpioCtrlRegs.GPAMUX2.bit.GPIO21 = 1;// 0=GPIO,  1=EQEPB-1,  2=Resv,  3=COMP2OUT
					//GpioCtrlRegs.GPAMUX2.bit.GPIO23 = 1;// 0=GPIO,  1=EQEPI-1,  2=Resv,  3=LINRX-A
					GpioCtrlRegs.GPDGMUX1.bit.GPIO99 = 1;
					GpioCtrlRegs.GPDMUX1.bit.GPIO99 = 1;

					break;

				case 11:
					// enable internal pull-up for the eQEP selected pins
					GpioCtrlRegs.GPAPUD.bit.GPIO10 = 0;
					GpioCtrlRegs.GPAPUD.bit.GPIO11 = 0;
					GpioCtrlRegs.GPAPUD.bit.GPIO13 = 0;

					// synchronize inputs to SYSCLKOUT
					GpioCtrlRegs.GPAQSEL1.bit.GPIO10 = 0;
					GpioCtrlRegs.GPAQSEL1.bit.GPIO11 = 0;
					GpioCtrlRegs.GPAQSEL1.bit.GPIO13 = 0;

					// set muxes
					GpioCtrlRegs.GPAGMUX1.bit.GPIO10 = 1;
					GpioCtrlRegs.GPAMUX1.bit.GPIO10 = 1;
					GpioCtrlRegs.GPAGMUX1.bit.GPIO11 = 1;
					GpioCtrlRegs.GPAMUX1.bit.GPIO11 = 1;
					GpioCtrlRegs.GPAGMUX1.bit.GPIO13 = 1;
					GpioCtrlRegs.GPAMUX1.bit.GPIO13 = 1;

					break;

				case 12:
					// enable internal pull-up for the eQEP selected pins
					GpioCtrlRegs.GPAPUD.bit.GPIO20 = 0;
					GpioCtrlRegs.GPAPUD.bit.GPIO21 = 0;
					GpioCtrlRegs.GPAPUD.bit.GPIO13 = 0;

					// synchronize inputs to SYSCLKOUT
					GpioCtrlRegs.GPAQSEL2.bit.GPIO20 = 0;
					GpioCtrlRegs.GPAQSEL2.bit.GPIO21 = 0;
					GpioCtrlRegs.GPAQSEL1.bit.GPIO13 = 0;

					// set muxes
					GpioCtrlRegs.GPAGMUX2.bit.GPIO20 = 0;
					GpioCtrlRegs.GPAMUX2.bit.GPIO20 = 1;
					GpioCtrlRegs.GPAGMUX2.bit.GPIO21 = 0;
					GpioCtrlRegs.GPAMUX2.bit.GPIO21 = 1;
					GpioCtrlRegs.GPAGMUX1.bit.GPIO13 = 1;
					GpioCtrlRegs.GPAMUX1.bit.GPIO13 = 1;

					break;

				case 13:
					// enable internal pull-up for the eQEP selected pins
					GpioCtrlRegs.GPAPUD.bit.GPIO20 = 0;
					GpioCtrlRegs.GPAPUD.bit.GPIO11 = 0;
					GpioCtrlRegs.GPDPUD.bit.GPIO99 = 0;

					// synchronize inputs to SYSCLKOUT
					GpioCtrlRegs.GPAQSEL2.bit.GPIO20 = 0;
					GpioCtrlRegs.GPAQSEL1.bit.GPIO11 = 0;
					GpioCtrlRegs.GPDQSEL1.bit.GPIO99 = 0;

					// set muxes
					GpioCtrlRegs.GPAGMUX2.bit.GPIO20 = 0;
					GpioCtrlRegs.GPAMUX2.bit.GPIO20 = 1;
					GpioCtrlRegs.GPAGMUX1.bit.GPIO11 = 1;
					GpioCtrlRegs.GPAMUX1.bit.GPIO11 = 1;
					GpioCtrlRegs.GPDGMUX1.bit.GPIO99 = 1;
					GpioCtrlRegs.GPDMUX1.bit.GPIO99 = 1;

					break;

				case 14:
					// enable internal pull-up for the eQEP selected pins
					GpioCtrlRegs.GPAPUD.bit.GPIO20 = 0;
					GpioCtrlRegs.GPAPUD.bit.GPIO11 = 0;
					GpioCtrlRegs.GPAPUD.bit.GPIO13 = 0;

					// synchronize inputs to SYSCLKOUT
					GpioCtrlRegs.GPAQSEL2.bit.GPIO20 = 0;
					GpioCtrlRegs.GPAQSEL1.bit.GPIO11 = 0;
					GpioCtrlRegs.GPAQSEL1.bit.GPIO13 = 0;

					// set muxes
					GpioCtrlRegs.GPAGMUX2.bit.GPIO20 = 0;
					GpioCtrlRegs.GPAMUX2.bit.GPIO20 = 1;
					GpioCtrlRegs.GPAGMUX1.bit.GPIO11 = 1;
					GpioCtrlRegs.GPAMUX1.bit.GPIO11 = 1;
					GpioCtrlRegs.GPAGMUX1.bit.GPIO13 = 1;
					GpioCtrlRegs.GPAMUX1.bit.GPIO13 = 1;

					break;

				case 15:
					// enable internal pull-up for the eQEP selected pins
					GpioCtrlRegs.GPAPUD.bit.GPIO10 = 0;
					GpioCtrlRegs.GPAPUD.bit.GPIO11 = 0;
					GpioCtrlRegs.GPDPUD.bit.GPIO99 = 0;

					// synchronize inputs to SYSCLKOUT
					GpioCtrlRegs.GPAQSEL1.bit.GPIO10 = 0;
					GpioCtrlRegs.GPAQSEL1.bit.GPIO11 = 0;
					GpioCtrlRegs.GPDQSEL1.bit.GPIO99 = 0;

					// set muxes
					GpioCtrlRegs.GPAGMUX1.bit.GPIO10 = 1;
					GpioCtrlRegs.GPAMUX1.bit.GPIO10 = 1;
					GpioCtrlRegs.GPAGMUX1.bit.GPIO11 = 1;
					GpioCtrlRegs.GPAMUX1.bit.GPIO11 = 1;
					GpioCtrlRegs.GPDGMUX1.bit.GPIO99 = 1;
					GpioCtrlRegs.GPDMUX1.bit.GPIO99 = 1;

					break;

				case 16:
					// enable internal pull-up for the eQEP selected pins
					GpioCtrlRegs.GPAPUD.bit.GPIO10 = 0;
					GpioCtrlRegs.GPAPUD.bit.GPIO21 = 0;
					GpioCtrlRegs.GPDPUD.bit.GPIO99 = 0;

					// synchronize inputs to SYSCLKOUT
					GpioCtrlRegs.GPAQSEL1.bit.GPIO10 = 0;
					GpioCtrlRegs.GPAQSEL2.bit.GPIO21 = 0;
					GpioCtrlRegs.GPDQSEL1.bit.GPIO99 = 0;

					// set muxes
					GpioCtrlRegs.GPAGMUX1.bit.GPIO10 = 1;
					GpioCtrlRegs.GPAMUX1.bit.GPIO10 = 1;
					GpioCtrlRegs.GPAGMUX2.bit.GPIO21 = 0;
					GpioCtrlRegs.GPAMUX2.bit.GPIO21 = 1;
					GpioCtrlRegs.GPDGMUX1.bit.GPIO99 = 1;
					GpioCtrlRegs.GPDMUX1.bit.GPIO99 = 1;

					break;

				case 17:
					// enable internal pull-up for the eQEP selected pins
					GpioCtrlRegs.GPAPUD.bit.GPIO10 = 0;
					GpioCtrlRegs.GPAPUD.bit.GPIO21 = 0;
					GpioCtrlRegs.GPAPUD.bit.GPIO13 = 0;

					// synchronize inputs to SYSCLKOUT
					GpioCtrlRegs.GPAQSEL1.bit.GPIO10 = 0;
					GpioCtrlRegs.GPAQSEL2.bit.GPIO21 = 0;
					GpioCtrlRegs.GPAQSEL1.bit.GPIO13 = 0;

					// set muxes
					GpioCtrlRegs.GPAGMUX1.bit.GPIO10 = 1;
					GpioCtrlRegs.GPAMUX1.bit.GPIO10 = 1;
					GpioCtrlRegs.GPAGMUX2.bit.GPIO21 = 0;
					GpioCtrlRegs.GPAMUX2.bit.GPIO21 = 1;
					GpioCtrlRegs.GPAGMUX1.bit.GPIO13 = 1;
					GpioCtrlRegs.GPAMUX1.bit.GPIO13 = 1;

					break;
			}
			break;

		case 2:
			obj->qep = &EQep2Regs;
			CpuSysRegs.PCLKCR4.bit.EQEP2 = 1;
			switch(aPinSet)
			{
				default:
				case 20:
					// enable internal pull-up for the eQEP selected pins
					GpioCtrlRegs.GPBPUD.bit.GPIO54 = 0;
					GpioCtrlRegs.GPBPUD.bit.GPIO55 = 0;
					GpioCtrlRegs.GPBPUD.bit.GPIO57 = 0;

					// synchronize inputs to SYSCLKOUT
					GpioCtrlRegs.GPBQSEL2.bit.GPIO54 = 0;
					GpioCtrlRegs.GPBQSEL2.bit.GPIO55 = 0;
					GpioCtrlRegs.GPBQSEL2.bit.GPIO57 = 0;

					// set muxes
					GpioCtrlRegs.GPBGMUX2.bit.GPIO54 = 1;
					GpioCtrlRegs.GPBMUX2.bit.GPIO54 = 1;
					GpioCtrlRegs.GPBGMUX2.bit.GPIO55 = 1;
					GpioCtrlRegs.GPBMUX2.bit.GPIO55 = 1;
					GpioCtrlRegs.GPBGMUX2.bit.GPIO57 = 1;
					GpioCtrlRegs.GPBMUX2.bit.GPIO57 = 1;

					break;
					
					case 21:
				      // enable internal pull-up for the eQEP selected pins
				    GpioCtrlRegs.GPAPUD.bit.GPIO24 = 0;
				    GpioCtrlRegs.GPAPUD.bit.GPIO25 = 0;
				    GpioCtrlRegs.GPAPUD.bit.GPIO26 = 0;

				    // synchronize inputs to SYSCLKOUT
				    GpioCtrlRegs.GPAQSEL2.bit.GPIO24 = 0;
				    GpioCtrlRegs.GPAQSEL2.bit.GPIO25 = 0;
				    GpioCtrlRegs.GPAQSEL2.bit.GPIO26 = 0;

				    // set muxes
				    GpioCtrlRegs.GPAGMUX2.bit.GPIO24 = 0;
				    GpioCtrlRegs.GPAMUX2.bit.GPIO24 = 2;
				    GpioCtrlRegs.GPAGMUX2.bit.GPIO25 = 0;
				    GpioCtrlRegs.GPAMUX2.bit.GPIO25 = 2;
				    GpioCtrlRegs.GPAGMUX2.bit.GPIO26 = 0;
				    GpioCtrlRegs.GPAMUX2.bit.GPIO26 = 2;

				    break;
			}
			break;

		case 3:
			obj->qep = &EQep3Regs;
			CpuSysRegs.PCLKCR4.bit.EQEP3 = 1;
			switch(aPinSet)
			{
				default:
				case 30:
					// enable internal pull-up for the eQEP selected pins
					GpioCtrlRegs.GPBPUD.bit.GPIO62 = 0;
					GpioCtrlRegs.GPBPUD.bit.GPIO63 = 0;
					GpioCtrlRegs.GPCPUD.bit.GPIO65 = 0;

					// synchronize inputs to SYSCLKOUT
					GpioCtrlRegs.GPBQSEL2.bit.GPIO62 = 0;
					GpioCtrlRegs.GPBQSEL2.bit.GPIO63 = 0;
					GpioCtrlRegs.GPCQSEL1.bit.GPIO65 = 0;

					// set muxes
					GpioCtrlRegs.GPBGMUX2.bit.GPIO62 = 1;
					GpioCtrlRegs.GPBMUX2.bit.GPIO62 = 1;
					GpioCtrlRegs.GPBGMUX2.bit.GPIO63 = 1;
					GpioCtrlRegs.GPBMUX2.bit.GPIO63 = 1;
					GpioCtrlRegs.GPCGMUX1.bit.GPIO65 = 1;
					GpioCtrlRegs.GPCMUX1.bit.GPIO65 = 1;

					break;
			}
			break;
	}

	obj->qep->QDECCTL.all = 0;

	obj->qep->QPOSINIT = aParams->QPOSINIT ;   // set counter initialization to 0
	obj->qep->QPOSMAX = aParams->QPOSMAX; //<<<< set maximum count value to 1999

	obj->qep->QEPCTL = aParams->QEPCTL;
	/*
	obj->qep->QEPCTL.bit.FREE_SOFT = 0; // all QEP timers stop on emulation suspend
	obj->qep->QEPCTL.bit.PCRM = 1; // operate QEP in Reset on Max counter mode
	obj->qep->QEPCTL.bit.SEI = 0; // disable strobe init
	obj->qep->QEPCTL.bit.IEI = 0; // disable index init
	obj->qep->QEPCTL.bit.SWI = 0; // don't allow software to initialize counter
	obj->qep->QEPCTL.bit.SEL = 0; // disable strobe event
	obj->qep->QEPCTL.bit.IEL = 3; //<<<< Latch on index event marker
	obj->qep->QEPCTL.bit.QPEN = 1; //<<<< QEP enable
	obj->qep->QEPCTL.bit.QCLM = 0; // disable EQEP capture latch
	obj->qep->QEPCTL.bit.UTE = 0; // disable unit timer
	obj->qep->QEPCTL.bit.WDE = 0; // disable QEP watchdog
    */
	obj->qep->QPOSCTL.all = 0;   // set eQEP position-compare control unit to the default mode

	obj->qep->QEINT.bit.QDC = 0;   // disable quadrature direction change interrupt
	obj->qep->QEINT.bit.PCO = 0;   // disable position counter overflow interrupt
	obj->qep->QEINT.bit.PCU = 0;   // disable position counter underflow interrupt
	obj->qep->QEINT.bit.PCM = 0;   // disable position compare match interrupt
	obj->qep->QEINT.bit.UTO = 0;   // disable unit time out event interrupt

	EDIS;
//#endif
}
